@@ -23,6 +23,7 @@
 #include "config.h"
 #endif
 
+#include <unistd.h>
 #include <glib/gi18n.h>
 #include <string.h>
 
@@ -34,7 +35,7 @@ static const struct {
   gchar *label,
 	*pixmap;
 } pix[] = {
-  { "cd",         "media-cdrom"         	},
+  { "cd",         "media-optical"         	},
   { "line",       "gvc-line-in"                 },
   { "mic",        "audio-input-microphone"      },
   { "mix",        "multimedia-volume-control"   },
@@ -61,7 +62,6 @@ cb_mute_toggled (GnomeVolumeControlButton *button,
 
   gst_mixer_set_mute (ctrl->mixer, ctrl->track,
 		      !gnome_volume_control_button_get_active (button));
-  gnome_volume_control_volume_sync (GNOME_VOLUME_CONTROL_VOLUME (ctrl->sliderbox));
 }
 
 static void
@@ -82,7 +82,7 @@ should_toggle_record_switch (const GstMixerTrack *track)
 {
   return GST_MIXER_TRACK_HAS_FLAG (track, GST_MIXER_TRACK_INPUT);
 }
- 
+
 
 static void
 cb_toggle_changed (GtkToggleButton *button,
@@ -116,32 +116,35 @@ cb_option_changed (GtkComboBox *box,
  * Timeout to check for changes in mixer outside ourselves.
  */
 
-static gboolean
-cb_check (gpointer data)
+void
+gnome_volume_control_track_update (GnomeVolumeControlTrack *trkw)
 {
-  GnomeVolumeControlTrack *trkw = data;
   gboolean mute, record;
   gboolean vol_is_zero = FALSE, slider_is_zero = FALSE;
 
+  g_return_if_fail (trkw != NULL);
+
   /* trigger an update of the mixer state */
   if (!GST_IS_MIXER_OPTIONS (trkw->track)) {
     gint *dummy = g_new (gint, MAX (trkw->track->num_channels, 1));
     gst_mixer_get_volume (trkw->mixer, trkw->track, dummy);
     g_free (dummy);
   }
-  
+
   mute = GST_MIXER_TRACK_HAS_FLAG (trkw->track,
 				GST_MIXER_TRACK_MUTE) ? TRUE : FALSE;
   record = GST_MIXER_TRACK_HAS_FLAG (trkw->track,
 				GST_MIXER_TRACK_RECORD) ? TRUE : FALSE;
 
   if (trkw->sliderbox) {
+    gnome_volume_control_volume_update (GNOME_VOLUME_CONTROL_VOLUME (trkw->sliderbox));
     gnome_volume_control_volume_ask (
       GNOME_VOLUME_CONTROL_VOLUME (trkw->sliderbox),
       &vol_is_zero, &slider_is_zero);
   }
+
   if (!slider_is_zero && vol_is_zero)
-    mute = TRUE;
+    mute |= TRUE;
 
   if (trkw->mute) {
     if (gnome_volume_control_button_get_active (trkw->mute) == mute) {
@@ -168,6 +171,13 @@ cb_check (gpointer data)
   /* FIXME:
    * - options.
    */
+}
+
+
+static gboolean
+cb_check (gpointer data)
+{
+  gnome_volume_control_track_update (data);
 
   return TRUE;
 }
@@ -189,6 +199,10 @@ gnome_volume_control_track_add_title (GtkTable *table,
   gchar *ulabel = NULL;
   gchar *str = NULL;
   gint i;
+  gboolean need_timeout = TRUE;
+
+  need_timeout = ((gst_mixer_get_mixer_flags (GST_MIXER (mixer)) &
+		   GST_MIXER_FLAG_AUTO_NOTIFICATIONS) == 0);
 
   /* start */
   ctrl = g_new0 (GnomeVolumeControlTrack, 1);
@@ -200,7 +214,8 @@ gnome_volume_control_track_add_title (GtkTable *table,
   ctrl->visible = TRUE;
   ctrl->table = table;
   ctrl->pos = tab_pos;
-  ctrl->id = g_timeout_add (200, cb_check, ctrl);
+  if (need_timeout)
+    ctrl->id = g_timeout_add (200, cb_check, ctrl);
 
   /* find image from label string (optional) */
   if (g_object_class_find_property (G_OBJECT_GET_CLASS (track), "untranslated-label"))
@@ -269,8 +284,7 @@ gnome_volume_control_track_add_title (GtkTable *table,
 static void
 gnome_volume_control_track_put_switch (GtkTable *table,
 				       gint      tab_pos,
-				       GnomeVolumeControlTrack *ctrl,
-				       GtkStatusbar *statusbar)
+				       GnomeVolumeControlTrack *ctrl)
 {
   GtkWidget *button;
   AtkObject *accessible;
@@ -287,7 +301,7 @@ gnome_volume_control_track_put_switch (GtkTable *table,
   msg = g_strdup_printf (_("Mute/unmute %s"), ctrl->track->label);
   button = gnome_volume_control_button_new ("audio-volume-high",
 					    "audio-volume-muted",
-					    statusbar, msg);
+					     msg);
   ctrl->mute = GNOME_VOLUME_CONTROL_BUTTON (button);
   g_free (msg);
   gnome_volume_control_button_set_active (GNOME_VOLUME_CONTROL_BUTTON (button),
@@ -317,8 +331,7 @@ gnome_volume_control_track_add_playback	(GtkTable *table,
 					 GstMixer *mixer,
 					 GstMixerTrack *track,
 					 GtkWidget *l_sep,
-					 GtkWidget *r_sep,
-					 GtkStatusbar *statusbar)
+					 GtkWidget *r_sep)
 {
   GnomeVolumeControlTrack *ctrl;
   GtkWidget *slider;
@@ -334,20 +347,19 @@ gnome_volume_control_track_add_playback	(GtkTable *table,
 
   /* switch exception (no sliders) */
   if (track->num_channels == 0) {
-    gnome_volume_control_track_put_switch (table, tab_pos, ctrl, statusbar);
+    gnome_volume_control_track_put_switch (table, tab_pos, ctrl);
     return ctrl;
   }
 
   ctrl->sliderbox = gnome_volume_control_volume_new (ctrl->mixer,
-						     ctrl->track, 6,
-						     statusbar);
+						     ctrl->track, 6);
   gtk_table_attach (GTK_TABLE (table), ctrl->sliderbox,
 		    tab_pos, tab_pos + 1, 2, 3,
 		    GTK_EXPAND, GTK_EXPAND | GTK_FILL, 0, 0);
   gtk_widget_show (ctrl->sliderbox);
 
   /* mute button */
-  gnome_volume_control_track_put_switch (table, tab_pos, ctrl, statusbar);
+  gnome_volume_control_track_put_switch (table, tab_pos, ctrl);
 
   return ctrl;
 }
@@ -358,8 +370,7 @@ gnome_volume_control_track_add_recording (GtkTable *table,
 					  GstMixer *mixer,
 					  GstMixerTrack *track,
 					  GtkWidget *l_sep,
-					  GtkWidget *r_sep,
-					  GtkStatusbar *statusbar)
+					  GtkWidget *r_sep)
 {
   GnomeVolumeControlTrack *ctrl;
   GtkWidget *button;
@@ -367,8 +378,7 @@ gnome_volume_control_track_add_recording (GtkTable *table,
   gchar *accessible_name, *msg;
 
   ctrl = gnome_volume_control_track_add_playback (table, tab_pos, mixer,
-						  track, l_sep, r_sep,
-						  statusbar);
+						  track, l_sep, r_sep);
   if (track->num_channels == 0) {
     return ctrl;
   }
@@ -379,8 +389,7 @@ gnome_volume_control_track_add_recording (GtkTable *table,
    * - there's something fishy about this button, it
    *     is always FALSE.
    */
-  button = gnome_volume_control_button_new ("audio-input-microphone", "audio-input-microphone-muted",
-					    statusbar, msg);
+  button = gnome_volume_control_button_new ("audio-input-microphone", "audio-input-microphone-muted", msg);
   ctrl->record = GNOME_VOLUME_CONTROL_BUTTON (button);
   g_free (msg);
   gnome_volume_control_button_set_active (GNOME_VOLUME_CONTROL_BUTTON (button),
@@ -412,8 +421,7 @@ gnome_volume_control_track_add_switch (GtkTable *table,
 				       GstMixer *mixer,
 				       GstMixerTrack *track,
 				       GtkWidget *l_sep,
-				       GtkWidget *r_sep,
-				       GtkStatusbar *statusbar)
+				       GtkWidget *r_sep)
 {
   GnomeVolumeControlTrack *ctrl;
 
@@ -437,8 +445,8 @@ gnome_volume_control_track_add_switch (GtkTable *table,
   g_signal_connect (ctrl->toggle, "toggled",
 		    G_CALLBACK (cb_toggle_changed), ctrl);
   gtk_widget_show (ctrl->toggle);
-                                                                                
-  return ctrl;  
+
+  return ctrl;
 }
 
 GnomeVolumeControlTrack *
@@ -447,8 +455,7 @@ gnome_volume_control_track_add_option (GtkTable *table,
 				       GstMixer *mixer,
 				       GstMixerTrack *track,
 				       GtkWidget *l_sep,
-				       GtkWidget *r_sep,
-				       GtkStatusbar *statusbar)
+				       GtkWidget *r_sep)
 {
   GnomeVolumeControlTrack *ctrl;
   GstMixerOptions *options = GST_MIXER_OPTIONS (track);
@@ -464,12 +471,18 @@ gnome_volume_control_track_add_option (GtkTable *table,
 
   /* optionmenu */
   active_opt = gst_mixer_get_option (mixer, GST_MIXER_OPTIONS (track));
-  ctrl->options = gtk_combo_box_new_text ();
-  opts = gst_mixer_options_get_values (options);
-  for (opt = opts; opt != NULL; opt = opt->next, i++) {
-    gtk_combo_box_append_text (GTK_COMBO_BOX (ctrl->options), opt->data);
-    if (!strcmp (active_opt, opt->data)) {
-      gtk_combo_box_set_active (GTK_COMBO_BOX (ctrl->options), i);
+  if (active_opt != NULL) {
+    ctrl->options = gtk_combo_box_new_text ();
+    opts = gst_mixer_options_get_values (options);
+    for (opt = opts; opt != NULL; opt = opt->next, i++) {
+      if (opt->data == NULL)
+	continue;
+
+      gtk_combo_box_append_text (GTK_COMBO_BOX (ctrl->options), opt->data);
+
+      if (g_str_equal (active_opt, opt->data)) {
+	gtk_combo_box_set_active (GTK_COMBO_BOX (ctrl->options), i);
+      }
     }
   }
 
